@@ -8,9 +8,12 @@ from sqlalchemy.engine.reflection import cache
 import requests
 import polars as pd
 import duckdb
-from duckdb_engine import DuckDBIdentifierPreparer
-from typing import Any, Dict, Optional, Tuple
+from duckdb_engine import DuckDBIdentifierPreparer, Dialect
+from typing import Any, Dict, Optional, Tuple, Type
 import sqlalchemy_dhis2 as module
+from sqlalchemy_dhis2.connection import add_authorization
+from sqlalchemy_dhis2.constants import _HEADER
+from sqlalchemy_dhis2.exceptions import DatabaseHTTPError
 
 duckdb_version: str = duckdb.__version__
 supports_attach: bool = duckdb_version >= "0.7.0"
@@ -30,17 +33,29 @@ class DBAPI:
     def Binary(x: Any) -> Any:
         return x
     
-class JSONHTTPDialect(DefaultDialect):
+class JSONHTTPDialect(Dialect):
     name = "dhis2"
     scheme = "http"
-    driver = "rest"
+    driver = "duckdb_engine"
     supports_statement_cache = False
     identifier_preparer: DuckDBIdentifierPreparer
     
+    #def __init__(self, *args: Any, **kwargs: Any) -> None:
+    #    kwargs["use_native_hstore"] = False
+    #    super().__init__(*args, **kwargs)
+        
     @classmethod
     # pylint: disable=method-hidden
     def dbapi(cls) -> ModuleType:
         return module
+        
+    #@staticmethod
+    #def dbapi(**kwargs: Any) -> Type[DBAPI]:
+    #    return DBAPI
+    
+    #@classmethod
+    #def import_dbapi(cls: Type["Dialect"]) -> Type[DBAPI]:
+    #    return cls.dbapi()
     
     def create_connect_args(self, url):
         #
@@ -48,8 +63,64 @@ class JSONHTTPDialect(DefaultDialect):
         #
         opts = url.translate_connect_args()
         opts.update(url.query)
+        #opts = url.translate_connect_args(database="database")
+        #opts["url_config"] = dict(url.query)
+        #user = opts["url_config"].pop("user", None)
+        #if user is not None:
+        #    opts["database"] += f"?user={user}"
         return ([], opts)
 
+    def connect(self, *cargs: Any, **cparams: Any) -> Any:
+        print(f"####{cargs}###::::{cparams}")
+        host = cparams.get('host')
+        database = cparams.get('database')
+        collection= cparams.get('collection','resources')
+        port=cparams.get('port',443)
+        username=cparams.get('username'),
+        password=cparams.get('password'),
+        use_ssl= True,
+        verify_ssl=None,
+        token=None,
+        auth=None,
+        duckdb_path=":memory:",
+        session = requests.Session()        
+        # Save to DuckDB
+        conn = duckdb.connect(duckdb_path)
+        
+        # by default session.verify is set to True
+        if verify_ssl is not None and verify_ssl in ["False", "false"]:
+            session.verify = False
+
+        if use_ssl in ["True", "true",True]:
+            proto = "https://"
+        else:
+            proto = "http://"
+        if collection is not None:
+            local_url = f"/api/{collection}"
+            if database is not None:
+                local_url = f"/{database}/api/{collection}"
+
+            add_authorization(session, username, password, token)
+            response = session.get(
+                f"{proto}{host}:{port}{local_url}",
+                headers=_HEADER,
+            )
+            if response.status_code != 200:
+                raise DatabaseHTTPError(response.text, response.status_code)
+                # Convert to Pandas DataFrame
+            df = pd.DataFrame(response.json())
+
+
+            conn.register("temp_table", df)
+            conn.execute(f"DROP TABLE IF EXISTS {collection}")
+            conn.execute(f"CREATE TABLE { collection } AS SELECT * FROM temp_table")
+            conn.unregister("temp_table")
+
+        return super().connect(*cargs, **cparams)
+
+    def on_connect(self) -> None:
+        pass
+    
     def execute_json_query(self, url, auth, duckdb_path, table_name):
         """
         Fetch JSON data and save it into DuckDB.
@@ -72,11 +143,11 @@ class JSONHTTPDialect(DefaultDialect):
         conn.unregister("temp_table")
 
         return conn
-
+    """
     def execute(self, cursor, statement, parameters, context=None):
-        """
+        
         Override execute to handle fetching JSON and saving to DuckDB.
-        """
+        
         print(f"####FG{statement}###{parameters}")
         conn = context["dialect"].execute_json_query(
             url=parameters["url"],
@@ -85,89 +156,16 @@ class JSONHTTPDialect(DefaultDialect):
             table_name=parameters["table_name"],
         )
         return conn.execute(statement)
-    
+    """
+        
     def do_ping(self,conn: Any ) -> bool:
         return True
 
     @cache  # type: ignore[call-arg]
     def get_table_names(self, connection: "Connection", schema=None, **kw: "Any"):  # type: ignore[no-untyped-def]
-        """
-        Return unquoted database_name.schema_name unless either contains spaces or double quotes.
-        In that case, escape double quotes and then wrap in double quotes.
-        SQLAlchemy definition of a schema includes database name for databases like SQL Server (Ex: databasename.dbo)
-        (see https://docs.sqlalchemy.org/en/20/dialects/mssql.html#multipart-schema-names)
-        """
-
-        if not supports_attach:
-            return super().get_table_names(connection, schema, **kw)
-
-        s = """
-            SELECT database_name, schema_name, table_name
-            FROM duckdb_tables()
-            WHERE schema_name NOT LIKE 'pg\\_%' ESCAPE '\\'
-            """
-        sql, params = self._build_query_where(schema_name=schema)
-        s += sql
-        rs = connection.execute(text(s), params)
-
-        return [
-            table
-            for (
-                db,
-                sc,
-                table,
-            ) in rs
-        ]
+        return super().get_table_names(connection, schema, **kw)
 
 
     @cache  # type: ignore[call-arg]
     def get_schema_names(self, connection: "Connection", **kw: "Any"):  # type: ignore[no-untyped-def]
-        """
-        Return unquoted database_name.schema_name unless either contains spaces or double quotes.
-        In that case, escape double quotes and then wrap in double quotes.
-        SQLAlchemy definition of a schema includes database name for databases like SQL Server (Ex: databasename.dbo)
-        (see https://docs.sqlalchemy.org/en/20/dialects/mssql.html#multipart-schema-names)
-        """
-
-        if not supports_attach:
-            return super().get_schema_names(connection, **kw)
-
-        s = """
-            SELECT database_name, schema_name AS nspname
-            FROM duckdb_schemas()
-            WHERE schema_name NOT LIKE 'pg\\_%' ESCAPE '\\'
-            ORDER BY database_name, nspname
-            """
-        rs = connection.execute(text(s))
-
-        qs = self.identifier_preparer.quote_schema
-        return [qs(".".join(nspname)) for nspname in rs]
-    
-    def _build_query_where(
-        self,
-        table_name: Optional[str] = None,
-        schema_name: Optional[str] = None,
-        database_name: Optional[str] = None,
-    ) -> Tuple[str, Dict[str, str]]:
-        sql = ""
-        params = {}
-
-        # If no database name is provided, try to get it from the schema name
-        # specified as "<db name>.<schema name>"
-        # If only a schema name is found, database_name will return None
-        if database_name is None and schema_name is not None:
-            database_name, schema_name = self.identifier_preparer._separate(schema_name)
-
-        if table_name is not None:
-            sql += "AND table_name = :table_name\n"
-            params.update({"table_name": table_name})
-
-        if schema_name is not None:
-            sql += "AND schema_name = :schema_name\n"
-            params.update({"schema_name": schema_name})
-
-        if database_name is not None:
-            sql += "AND database_name = :database_name\n"
-            params.update({"database_name": database_name})
-
-        return sql, params
\ No newline at end of file
+       return super().get_schema_names(connection, **kw)
\ No newline at end of file
